import type { Express } from "express";
import { createServer, type Server } from "http";
import { storage } from "./storage";
import { insertCartItemSchema, insertContactSchema, insertNewsletterSchema } from "@shared/schema";
import { ZodError } from "zod";

export async function registerRoutes(app: Express): Promise<Server> {
  // Products
  app.get("/api/products", async (_req, res) => {
    const products = await storage.getProducts();
    res.json(products);
  });

  app.get("/api/products/:id", async (req, res) => {
    const product = await storage.getProduct(parseInt(req.params.id));
    if (!product) {
      res.status(404).json({ message: "Product not found" });
      return;
    }
    res.json(product);
  });

  // Cart
  app.get("/api/cart", async (req, res) => {
    const sessionId = req.sessionID;
    const items = await storage.getCartItems(sessionId);
    res.json(items);
  });

  app.post("/api/cart", async (req, res) => {
    try {
      const sessionId = req.sessionID;
      const item = insertCartItemSchema.parse({ ...req.body, sessionId });
      const newItem = await storage.addCartItem(item);
      res.status(201).json(newItem);
    } catch (error) {
      if (error instanceof ZodError) {
        res.status(400).json({ message: "Invalid cart item data", errors: error.errors });
        return;
      }
      throw error;
    }
  });

  app.delete("/api/cart/:id", async (req, res) => {
    await storage.removeCartItem(parseInt(req.params.id));
    res.status(204).send();
  });

  app.patch("/api/cart/:id", async (req, res) => {
    const { quantity } = req.body;
    if (typeof quantity !== "number" || quantity < 1) {
      res.status(400).json({ message: "Invalid quantity" });
      return;
    }
    
    try {
      const updatedItem = await storage.updateCartItemQuantity(
        parseInt(req.params.id),
        quantity
      );
      res.json(updatedItem);
    } catch (error) {
      if (error instanceof Error) {
        res.status(404).json({ message: error.message });
        return;
      }
      throw error;
    }
  });

  // Newsletter
  app.post("/api/newsletter", async (req, res) => {
    try {
      const subscription = insertNewsletterSchema.parse(req.body);
      const newSubscription = await storage.addNewsletterSubscription(subscription);
      res.status(201).json(newSubscription);
    } catch (error) {
      if (error instanceof ZodError) {
        res.status(400).json({ message: "Invalid email", errors: error.errors });
        return;
      }
      throw error;
    }
  });

  // Contact
  app.post("/api/contact", async (req, res) => {
    try {
      const contact = insertContactSchema.parse(req.body);
      const newContact = await storage.createContact(contact);
      res.status(201).json(newContact);
    } catch (error) {
      if (error instanceof ZodError) {
        res.status(400).json({ message: "Invalid contact data", errors: error.errors });
        return;
      }
      throw error;
    }
  });

  const httpServer = createServer(app);
  return httpServer;
}
